/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.client;

import javax.servlet.ServletContext;
import javax.servlet.http.*;
import org.apache.log4j.Logger;
import shocks.dawp.*;

/**
 * <p>The primary support class for action components.  This class provides a 
 * collection of support methods and gives implementing subclasses as much 
 * information about the servlet environment as possible.  Action classes can
 * get the <code>Request</code>, <code>Response</code> and 
 * <code>ServletContext</code> objects as well as metadata information about
 * the current workflow sequence (which could include information from any 
 * filters that executed during the "downswing" of the workflow sequence).</p>
 * 
 * <p>All actions (except those which extend FilterSupport and represent
 * crosscutting concerns of the business logic, such as security, i18n and the
 * like) must extend this class. Specifically, <strong>all <code>Action</code>
 * classes must override the <code>execute(HttpServletRequest, 
 * HttpServletResponse)</code> method</strong>.
 * 
 */
public abstract class ActionSupport implements Action, Cloneable {
	private Logger log = Logger.getLogger("Action]");
	private String className;
	private ServletContext ctx;
	
	public final void init(DataSource metadata){
		this.className = (String)metadata.getAttribute("className");
	}
	
	public final String getClassName() {
		return this.className;
	}
	
	public final ServletContext getContext() {
		return this.ctx;
	}
	
	/**
	 * <p><strong>This method cannot be overridden by the user</strong>. It
	 * is specifically designed for use by the container.  The method the user
	 * should concern themselves with is below.</p>
	 * 
	 * <p>This method is called by the enclosing workflow component, usually a
	 * <code>SequenceTerminator</code>.  Its purpose is to staff the action
	 * with a request & response pair, and a reference to the current servlet
	 * context.</p>
	 * 
	 * @return DataSource
	 */
	public final DataSource execute(
		HttpServletRequest request, 
		HttpServletResponse response, 
		ServletContext ctx) {
			
		this.ctx = ctx;
		return execute(request, response);
		//getResult(request, response, ctx);
	}
	
	/**
	 * This method must be overridden by <code>Action</code> authors and must 
	 * contain a <code>Result</code> object containing instructions for the 
	 * <code>WorkflowController</code>, so it can determine which 
	 * <code>Sequence</code> to perform next.  Also, the <code>Result</code>
	 * object can contain instructions for filters which might execute during
	 * the "upswing" of the workflow sequence.</p> 
	 * 
	 * @param request
	 * @param response
	 * @return Result
	 */
	public abstract Result execute(
		HttpServletRequest request, 
		HttpServletResponse response);
		
	/**
	 * Returns a clone of the object identical in all regards save its location
	 * in memory.
	 */
	public final Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			throw new Error("Could not clone the ActionSupport object");
		}
	}
}