/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.client;

import java.util.HashMap;
import java.util.Map;

import shocks.dawp.DataSource;

/**
 * DataSource containing information about the execution of a target <code>Action</code>.
 * 
 * <blockquote><strong>result</strong>
 * <br/><strong>(n.)</strong> 3.b. The effect, consequence, issue, or outcome 
 * of some action, process, design, etc;
 * <br/><strong>(v.)</strong> [f. re- RE- + <em>saltare</em> 
 * to leap.] 2.c. To recoil; to rebound or spring back. <em>Obs.</em>
 * <br/>-- Oxford English Dictionary</blockquote>
 * 
 * <p>Result objects are a type of <code>DataSource</code> used by the 
 * container to carry information about an <code>Action</code> back up 
 * the workflow sequence to the <code>WorkflowController</code>.</p> 
 * 
 * <p>This information <strong>must</strong> include a "result" string 
 * corresponding to one of the "name" attributes from the 
 * <code>shocks-workflow.xml</code> file (see below), 
 * but may also include any of the following:
 * <li/>instructions about whether or not the target <code>Action</code> was 
 * successful in its execution
 * <li/>Errors
 * <li/>Exceptions
 * <li/>Custom, application-specific information about the <code>Action</code>
 * </p>
 * 
 * <p>The <code>ActionSupport</code> implementation, or "target action" ideally 
 * doesn't make assumptions about which object, if any, will make use of the 
 * contents of its <code>Result</code> object.  It worries about its own end
 * and lets the <code>WorkflowController</code> and applicable 
 * <code>FilterSupport</code> implementations handle the <code>Result</code>.
 * 
 * <p>On its way back up the workflow sequence, the <code>Result</code> object 
 * can be passed into system-level actions (<code>FilterSupport</code> 
 * implementations), which can use it to perform common operations like 
 * logging, i18n, <code>Error</code> and <code>Exception</code> handling, 
 * system-level testing, custom alerts, or handle information from the 
 * target <code>Action</code> which are application specific but system-wide.
 * </p>
 *
 */
public class Result implements DataSource {
	private Map map = new HashMap();
	
	/**
	 * The String parameter for the constructor is the only required data for
	 * a valid <code>Result</code>. This string <strong>must</strong> 
	 * correspond to one of the "name" attributes of a "result" element associated
	 * with the currently executing action from the <code>shocks-workflow.xml</code> 
	 * descriptor file.
	 * 
	 * @param result
	 */
	public Result(String result) {
		map.put("result", result);
	}
	
	public void setAttribute(String key, Object value) {
		map.put(key, value);
	}
	
	public Object getAttribute(String key) {
		return map.get(key);
	}
}
