/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;
import org.dom4j.Document;

import shocks.framework.meta.*;
import shocks.framework.workflow.filter.*;
import shocks.framework.workflow.sequence.*;
import shocks.framework.action.*;
import shocks.framework.util.XMLLoader;

public class SystemLoader {
	private static Logger log;
	private static MetadataRepository metadata;
	private static FilterRepository filters;
	private static ActionRepository actions;
	private static SequenceRepository sequences;

	public static void load(ServletContext ctx) {
		log = Logger.getLogger("Shocks][SysLoader");
		String path = ctx.getRealPath("/WEB-INF/shocks-workflow.xml");
		log.debug("workflow descriptor path: " + path);
		Document doc = XMLLoader.getDocument(path);
		loadConfigurationData(doc, ctx);
		metadata = MetadataRepositoryFactory.load(doc, ctx);
		if(metadata != null) {
			actions = ActionRepositoryFactory.load(ctx);
			if(actions != null) {
				filters = FilterRepositoryFactory.load(ctx);
				if(filters != null) {
					sequences = SequenceRepositoryFactory.load(ctx);
					if(sequences != null) {
						log.info(ctx.getAttribute("shocks.appName") + 
							" module successfully loaded");
					}
				}
			}
		}
	}
	
	private static void loadConfigurationData(
		Document doc,
		ServletContext ctx) {

		log.info("Loading the shocks-workflow.xml document");
		if (doc == null) {
			log.info("The document is null.  I won't be able to load the "
				+ "config settings. ");
		} else {
			ctx.setAttribute(
				"shocks.appName",
				doc.valueOf("/workflow/@application"));
			ctx.setAttribute(
				"shocks.actionSuffix",
				doc.valueOf("/workflow/@actionSuffix"));
			ctx.setAttribute(
				"shocks.viewSuffix",
				doc.valueOf("/workflow/@viewSuffix"));
			ctx.setAttribute(
				"shocks.defaultVersion",
				doc.valueOf("/workflow/@version"));
		}
	}
}
