/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.action.repository;

import java.util.*;
import javax.servlet.*;
import javax.management.*;
import shocks.client.*;
import shocks.dawp.*;
import shocks.framework.util.*;
import shocks.framework.action.*;
import shocks.framework.meta.*;
import org.apache.log4j.*;

/**
 * Actions are loaded in a very specific manner.  The first step is to get
 * all of the action metadata from this application's MetadataRepository.
 * 
 * Get all of the relevant metadata beans from the MetadataRepository.
 * Cross reference all of those beans in the ActionRepository.
 * 
 * Any action metadata beans which have no corresponding action in the action
 * repository must be loaded from the local classloader.
 * 
 * Actions which cannot be loaded from the local classloader and do not exist
 * in the ActionRepository should be identified in the logs for debugging.
 */
public class CentralActionRepositoryLoader {
	public static void load(ServletContext ctx){
		Logger log = Logger.getLogger("ActionLoader");
		MBeanServer server = MBeanServerBridge.getServer();
		MetadataRepository mdr = MetadataRepositoryFactory.getRepository(ctx);
		Set actions = mdr.getTypeSet("action");
		
		Iterator iter = actions.iterator();
		while(iter.hasNext()) {
			DataSource metadata = (DataSource)iter.next();
			ActionRepository actionRepository = 
				ActionRepositoryFactory.getRepository(ctx);
			Action action = actionRepository.leaseAction(metadata);
			if(action == null) {
				action = registerNewAction(metadata, actionRepository, ctx);
			}
		}
	}
	
	private static Action registerNewAction(
		DataSource metadata,
		ActionRepository actionRepository,
		ServletContext ctx) {
		
		Logger log = Logger.getLogger("ActionLoader");
		Action action = buildAction(metadata, ctx);
		if(action != null) {
			actionRepository.addAction(action);
			return actionRepository.leaseAction(metadata);
		}
		return null;
	}
	
	/**
	 * This method assumes the action in question is an instance of ActionSupport.
	 * 
	 * @param metadata
	 * @param ctx
	 * @return
	 */
	private static Action buildAction(
		DataSource metadata,
		ServletContext ctx) {
		
		Logger log = Logger.getLogger("ActionLoader");
		String className = (String) metadata.getAttribute("className");
		Action action = null;
		
		try {
			ClassLoader loader = CentralActionRepositoryLoader.
				class.getClassLoader();
				
			Class c = loader.loadClass(className);
			action = (Action)c.newInstance();
			((ActionSupport)action).init(metadata);
			log.debug("Action class " + className + " = " + action );
			
		} catch (ClassNotFoundException e) {
			log.warn("The action class " + className + 
				" could not be loaded in this ClassLoader");
		
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return action;
	}
}