/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.meta.repository;

import java.util.HashMap;
import java.util.Map;

import shocks.dawp.DataSource;
import shocks.framework.meta.MetadataRepository;
import org.apache.log4j.*;
import java.util.*;

/**
 * The repository handles instance pooling and will return an instance of any 
 * MDBean type upon request.
 *
 */
public class CentralMetadataRepository implements 
	MetadataRepository, 
	CentralMetadataRepositoryMBean {
		
	private final Map elements = new HashMap();
	private Logger log = Logger.getLogger("Shocks][CMR");
	
	/**
	 * Returns an instance of a DataSource of a matching name and type.
	 * Returns null if no instance exists with matching attributes.
	 */
	public DataSource leaseDataSource(String type, String name) {
		MDBeanNameCache e = cache(type);
		if(e != null) {
			return e.lease(name);
		}
		return null;
	}
	
	/**
	 * Adds the DataSource instance to an InstancePool, sorted by the
	 * element's type and name. If no InstancePool exists for this type
	 * of DataSource, one is created.  If there already is an InstancePool
	 * with a matching name and type, an error will be logged.
	 * 
	 * @param instance -- DataSource to be pooled
	 */
	public void addDataSource(DataSource instance){
		// sort by types.
		String type = (String)instance.getAttribute("type");
		String name = (String)instance.getAttribute("name");
		
		//System.out.println("Adding " + type + " DataSource");
		MDBeanNameCache cache = cache(type);
		
		// if there is no cache of the same type, make a new one.
		if(cache == null) {
			cache = new MDBeanNameCache(type);
			// then add.
			cache.add(instance);
			elements.put(type, cache);
			
		// if a cache of the same type already exists, use that.
		} else {
			cache.add(instance);
		}
		log.info("Loaded " + type + " metadata, name=" + name);
	}
	
	/**
	 * Releases a DataSource back into the pool.
	 * 
	 * @param instance -- DataSource to be released back into the pool
	 */
	public void release(DataSource instance) {
		cache((String)instance.getAttribute("type")).release(instance);
	}
	
	public Set getTypeSet(String type) {
		MDBeanNameCache cache = cache(type);
		if(cache != null) {
			return cache(type).instanceSet();
		} else {
			return null;
		}
	}
	
	/**
	 * Returns the ElementStack whose type matches the <code>type</code>
	 * parameter.
	 * 
	 * @param type -- DataSource element type
	 */
	private MDBeanNameCache cache(String type) {
		try {
			return (MDBeanNameCache)elements.get(type);
		} catch (Exception e) {
			log.warn("There is no instance pool of type '" + type + "'.");
		}
		return null;
	}
}
