/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.meta.repository;

import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.Node;

import shocks.framework.meta.*;

/**
 * Parses xml file and binds the contents into Metadata bean (MDBean) objects,
 * which are then submitted to the Repository for later retrieval.
 * 
 * Unlike the other, simpler MetadataFactory, the JMXRepositoryLoader makes
 * use of a repository stored in the MBeanServer.  Because it allows multiple
 * application modules in different web archives to share workflow metadata
 * (and even Actions), it needs to take extra precautions while validating the
 * content and structure of the Metadata.  In particular, it must ensure that
 * there are not namespace issues with the MDBeans between application modules.
 * 
 * Because the metadata for different applications is stored in separate
 * repositories in the MBeanServer, there should be no namespace conflict
 * between applications.  However, there could be namespace issues between
 * different modules within the same application.
 * 
 * What this means is that a developer could deploy an application module web
 * archive which contained in its workflow descriptor a duplicate action
 * entry, new instructions for a old action or even a newer version of an 
 * existing action class.
 * 
 * In order to alter the behavior of an existant workflow system, the metadata
 * from the newer descriptor needs to be able to override the metadata from
 * the original descriptor.  The key to this lies in versioning.  If each
 * element in the workflow descriptor bears a version number, it should allow
 * the repository to manage the addition of new metadata resources by comparing
 * their version numbers, allowing newer versions to replace the older versions
 * and barring older or equivalent versions from overriding newer versions.
 * 
 * If we're cautious when we build this, we might be able to label and keep the
 * older versions, allowing the developer to roll back the application to a
 * prior functional state.
 * 
 * Invalid metadata should result in a fatal error at loadtime which must be
 * logged and presented to the developer.
 * 
 */
public class CentralMetadataRepositoryLoader {
	private static Logger log = Logger.getLogger("Shocks][CMRLoader");
	private static MetadataValidator validator;
	private static MetadataRepository repository;

	/**
	 * Process the incoming document, break it up by element type and 
	 * forward it to the element processing methods. Once the metadata is
	 * bound into the DataSource components and validated, load each component
	 * into the MetadataRepository.
	 * 
	 * @param ctx ServletContext reference
	 * @param document XML Document
	 */
	public static void load(ServletContext ctx, Document document) {

		Stack mdbeans = new Stack();
		repository = MetadataRepositoryFactory.getRepository(ctx);
		validator = new MetadataValidator(ctx);

		log.info("Binding the workflow metadata");
		String[] types = {"filter", "action", "security-role"};
		for(int i=0; i<types.length; i++) {
			mdbeans = bindElements(
				document.selectNodes("/workflow/" + types[i]),
				types[i], mdbeans);
		}

		Iterator iter = mdbeans.iterator();
		while (iter.hasNext()) {
			// And add whatever comes out to the repository.
			repository.addDataSource((MDBean) iter.next());
		}
	}

	/**
	 * Bind the filter elements and their attributes into a DataSource, then
	 * store the instance in the MetadataRepository.
	 * 
	 * @param elements
	 * @param type
	 * @param mdbeans
	 * @param mdbean
	 * @param node
	 */
	private static Stack bindElements(
		List elements,
		String type,
		Stack mdbeans) {
		MDBean mdbean;
		Node node;
		if (!elements.isEmpty()) {
			// For each Node in the list
			for (int i = 0; i < elements.size(); i++) {
				// Build a new MDBean and fill it with the XML metadata
				node = (Node) elements.get(i);
				mdbean = populateMDBean(node, type, i + 1);

				// Push the mdbean into a stack
				mdbeans.push(mdbean);

			}
			// Once the stack is done, push it through the validator.
			mdbeans = validator.checkStack(mdbeans);
		}
		return mdbeans;
	}

	/**
	 * Performs the task of populating the MDBean based on its element type.
	 * 
	 * @param type -- type of element currently being processed
	 * @param num -- number of the element (used in XPath query)
	 */
	private static MDBean populateMDBean(Node node, String type, int num) {
		MDBean mdbean = new MDBean();
		// This string is used later to generate the XPath queries.
		String nodeId = "/workflow/" + type + "[" + num + "]/";

		log.info("");
		log.info("Binding " + type + " metadata.");
		mdbean.setAttribute("type", type);

		if (type == "filter") {
			String[] attributes =
				{ "name", "incoming", "outgoing", "depends", "version" };
			return bindAttributes(mdbean, node, nodeId, attributes);

		} else if (type == "action") {
			String[] attributes = 
				{"name", "className", "permission", "filter", "managed", "version" };
			mdbean = bindAttributes(mdbean, node, nodeId, attributes);
			return bindActionResults(mdbean, node, nodeId, num);

		} else if (type == "security-role") {
			String[] attributes =
				{ "name", "includes", "description", "version" };
			return bindAttributes(mdbean, node, nodeId, attributes);

		} else {
			log.error("Descriptor does not support " + type + " elements.");
			return null;
		}
	}

	/**
	 * Binds the attributes to the DataSource.
	 * 
	 * @param mdbean
	 * @param node
	 * @param nodeId
	 * @param attributes
	 * @return
	 */
	private static MDBean bindAttributes(
		MDBean mdbean,
		Node node,
		String nodeId,
		String[] attributes) {
		for (int i = 0; i < attributes.length; i++) {
			if (attributes[i] == "version") {
				String value = node.valueOf(nodeId + "@" + attributes[i]);
				String s = validator.checkVersion(value);
				mdbean.setAttribute(attributes[i], s);
				
			} else if (attributes[i] == "className") {
				String value = node.valueOf(nodeId + "@class");
				String s = validator.checkVersion(value);
				mdbean.setAttribute(attributes[i], s);
				
			} else if (attributes[i] == "managed") {
				String value = node.valueOf(nodeId + "@managed");
				String s = validator.checkManaged(value);
				mdbean.setAttribute(attributes[i], s);
				
			} else {
				String value = node.valueOf(nodeId + "@" + attributes[i]);
				String s = validator.checkNullString(value);
				mdbean.setAttribute(attributes[i], s);
			}
			log.info(" @" + attributes[i] + " = " 
				+ (String) mdbean.getAttribute(attributes[i]));
		}
		return mdbean;
	}

	/**
	 * Unlike normal elements, result elements don't go into their own MDBean.
	 * Instead, they get stored inside the MDBean through its ActionResult
	 * interface.
	 * 
	 * @param results
	 */
	private static MDBean bindActionResults(
		MDBean mdbean,
		Node node,
		String nodeId,
		int iter) {

		List results = node.selectNodes(nodeId + "/result");
		if (!results.isEmpty()) {
			//for each result element in the list
			for (int i = 0; i < results.size(); i++) {
				int num = i + 1;
				String xpath = nodeId + "result[" + num + "]/";

				//set the attributes
				String resultName =
					validator.checkNullString(node.valueOf(xpath + "@name"));
				String resultNext =
					validator.checkNullString(node.valueOf(xpath + "@next"));
				mdbean.setResult(resultName, resultNext);

				log.info(" result @name = " + resultName);
				log.info(" result @next = " + resultNext);
			}
		}
		return mdbean;
	}
}