/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.meta.repository;

import java.util.HashMap;
import java.util.Map;

import shocks.dawp.DataSource;
import org.apache.log4j.*;
/**
 * Encapsulates XML metadata for the workflow engine. Because the structure of
 * the workflow.xml file is defined in XML Schema, we are able to assume a
 * great deal of knowledge about the content and structure of the XML file.
 * 
 * That allows us to construct "metadata beans" or MDBeans to bind the values
 * of the XML document into memory for fast retrieval.  We can also provide
 * custom solutions for the different types of elements in our workflow 
 * descriptor to increase performance and ease of use.
 * 
 * One of the requirements of the data contained in MDBeans is that it be 
 * provide sufficient information for looking up other MDBeans that it
 * references. This means if an &lt;action&gt; points at another &lt;action&gt,
 * it needs to specify the name and the type.
 *
 */
public class MDBean implements Cloneable, DataSource {
	private Map attributes = new HashMap();
	private Map actionResults = new HashMap();
	private Logger log = Logger.getLogger("MDBean");
	
	/* ===================================================================== */
	
	/**
	 * This returns the value of the attribute specified.
	 */
	public Object getAttribute(String key) {
		return (key == null || key == "" ? null : (String)attributes.get(key));
	}
	
	/**
	 * This binds the value of an attribute into memory.
	 */
	public void setAttribute (String key, Object value) {
		if (key == null || key == "") {
			log.error("MDBean.setAttribute: String key must not be null or blank");
		}else if (value == null || value == "") {
			log.error("MDBean.setAttribute: Object value must not be null or blank");
		}else if (attributes.containsKey(key)) {
			log.error("the " + key + " value has already been set and " +
				"can't be set for this " + getAttribute("type") + ".");
		} else {
			attributes.put(key, value);
		}
	}
	
	/**
	 * Returns a clone of the object identical in all regards save its location
	 * in memory.
	 */
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			throw new Error("Could not clone the MDBean");
		}
	}
	
	/**
	 * MDBeans which represent actions must provide a simple way to access 
	 * information about that action's results.  That information is stored
	 * here.  MDBeans representing other components needn't use this method.
	 */
	public void setResult(String resultName, String resultValue) {
		if (resultName != null && resultValue != null) {
			if(getAttribute("type") == "action") {
				actionResults.put(resultName, resultValue);
			}
		} else {
			//if this is not an <action> bean, fail gracefully & log the error.
			log.error(getAttribute("type") + " MDBeans cannot store results.");
		}
	}
	
	/**
	 * This returns the result string of an action.  
	 * Accepts the result name as an attribute and will return
	 * the contents of the "next" parameter from the workflow.xml file.
	 */
	public String getResult(String resultName) {
		if(resultName != null) {
			return (String)actionResults.get(resultName);
		}
		return null;
	}
}
