/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.meta.repository;

import java.util.*;
import org.apache.log4j.Logger;
import shocks.dawp.DataSource;

/**
 * ElementStack keeps a Collection of all the InstancePool objects with the
 * same element type but different names.  It manages the creation and storage
 * of InstancePools for the Repository as well as providing access to them.
 *
 */
class MDBeanNameCache {

	/**
	 * This signifies the type of elements this ElementStack is responsible
	 * for managing.
	 */
	protected final String ELEMENT_TYPE;
	private HashMap namedCaches = new HashMap();
	private Logger log = Logger.getLogger("Shocks][MDR");
	
	/**
	 * Returns an ElementStack whose responsibility it is to manage all
	 * InstancePools of a certain type.
	 * 
	 * @param type -- the assigned type of XML element to manage.
	 */
	public MDBeanNameCache(String type) {
		this.ELEMENT_TYPE = type;
	}
	
	/**
	 * Returns a DataSource object from the relevant InstancePool.  The
	 * DataSource will be marked in the InstancePool as "ready".  When
	 * you are finished with the instance, pass it to the <code>release
	 * </code> method, so it can be placed back into the pool.
	 * 
	 * @param name
	 * @return DataSource
	 */
	public DataSource lease(String name) {
		if(cache(name) != null) {
			return cache(name).lease();
		}
		log.warn("There is no cache with that name");
		return null;
	}
	
	/**
	 * @param instance -- the DataSource instance which needs pooling
	 */
	public synchronized void add(DataSource instance) {
		String name = (String)instance.getAttribute("name");
		if(cache(name) != null) {
			cache(name).add(instance);
		} else {
			namedCaches.put(name, new MDBeanVersionCache(instance));
		}
	}
	
	/**
	 * Releases a DataSource instance back into the InstancePool.
	 * 
	 * @param instance -- the instance to be released
	 */
	public synchronized void release(DataSource instance) {
		String name = (String)instance.getAttribute("name");
		if(cache(name) != null) {
			cache(name).release(instance);
		} else {
			log.warn("There are no DataSources named " + name);
		}
	}
	
	public Set instanceSet() {
		Set instances = new HashSet();
		Iterator i = namedCaches.keySet().iterator();
		while(i.hasNext()) {
			String key = (String)i.next();
			instances.add(cache(key).lease());
		}
		return instances;
	}
	
	private boolean correctType(DataSource instance) {
		return instance.getAttribute("type") == ELEMENT_TYPE;
	}
	
	private MDBeanVersionCache cache(String name) {
		return (MDBeanVersionCache)namedCaches.get(name);
	}
}
