/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.meta.repository;

import shocks.framework.meta.MetadataRepository;
import shocks.dawp.*;
import shocks.framework.util.*;
import javax.management.*;
import java.util.*;

/**
 * This class is a facade for the CentralMetadataRepository in the MBeanServer.
 */
public class MetadataRepositoryProxy implements MetadataRepository {
	private MBeanServer server;
	private ObjectName objName;

	public MetadataRepositoryProxy(ObjectName objName) {
		MBeanServer server = MBeanServerBridge.getServer();
		this.server = server;
		this.objName = objName;
	}

	public void addDataSource(DataSource metadata) {
		try {
			Object[] params = { metadata };
			String[] types = { "shocks.dawp.DataSource" };
			server.invoke(objName, "addDataSource", params, types);
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (MBeanException e) {
			e.printStackTrace();
		} catch (ReflectionException e) {
			e.printStackTrace();
		}
	}

	public DataSource leaseDataSource(String type, String name) {
		DataSource mdbean = null;
		try {
			Object[] params = { type, name };
			String[] types = { "java.lang.String", "java.lang.String" };
			mdbean = (DataSource) server.invoke(
				objName, "leaseDataSource", params, types);
			if (mdbean == null) {
				throw new InstanceNotFoundException();
			}
		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (MBeanException e) {
			e.printStackTrace();
		} catch (ReflectionException e) {
			e.printStackTrace();
		}
		return mdbean;
	}

	/**
	 * Release the DataSource back into the repository. This entails passing an
	 * invocation through the MBeanServer to release the DataSource.
	 * 
	 * @param metadata to be released back into the repository.
	 */
	public void release(DataSource metadata) {
		try {
			Object[] params = {metadata};
			String[] types = {"shocks.dawp.DataSource"};
			server.invoke(objName, "release", params, types );
				
		} catch(InstanceNotFoundException e) {
			e.printStackTrace();
		} catch(MBeanException e) {
			e.printStackTrace();
		} catch(ReflectionException e) {
			e.printStackTrace();
		}
	}
	
	public Set getTypeSet(String type) {
		try {
			Object[] params = {type};
			String[] types = {"java.lang.String"};
			return (Set)server.invoke(objName, "getTypeSet", params, types );
				
		} catch(InstanceNotFoundException e) {
			e.printStackTrace();
		} catch(MBeanException e) {
			e.printStackTrace();
		} catch(ReflectionException e) {
			e.printStackTrace();
		}
		return null;
	}
}
