/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.meta.repository;

import java.util.Iterator;
import java.util.Stack;

import javax.servlet.ServletContext;

import org.apache.log4j.Logger;

public final class MetadataValidator {
	private Logger log;
	private ServletContext ctx;
	private Stack outgoing;
	
	public MetadataValidator(ServletContext ctx) {
		log = Logger.getLogger("Shocks][Validator");
		if(ctx == null) {
			log.warn("Null ServletContext passed to constructor.");
		} else {
			this.ctx = ctx;
		}
	}
	
	/**
	 * Ensures there are no duplicate entries in the workflow descriptor and
	 * that each MDBean entered is sound of form.
	 * 
	 * @param incoming
	 * @return Stack
	 */
	public synchronized Stack checkStack(Stack incoming) {
		outgoing = new Stack();
		Stack names = new Stack();
		if(incoming != null) {
			Iterator i = incoming.iterator();
			while(i.hasNext()) {
				MDBean mdbean = (MDBean)i.next();
				String name = (String)mdbean.getAttribute("name");
				if(isWellFormed(mdbean)){
					if(!names.contains(name)) {
						names.push(name);
						outgoing.push(mdbean);
					} else {
						log.info("An MDBean named " + name + 
							" already exists in this module.  " +
							"The second instance will not be loaded.");
					}
				} else {
					log.info("The " + name + " MDBean contains invalid data " +
						"and cannot be loaded.");
				}
			}
		}
		return outgoing;
	}
	
	/**
	 * <p>Validates the contents of an MDBean.  
	 * 
	 * <p>Each type of MDBean has a specific set of requirements and a method 
	 * which we delegate to in order to determine its validity. MDBeans are
	 * tested in relation to one another in order to prevent duplicate
	 * entries.</p>
	 * 
	 * @param mdbean
	 * @param type
	 * @return
	 */
	private boolean isWellFormed(MDBean mdbean) {
		String type = (String)mdbean.getAttribute("type");
		if(type == "filter") { 
			return isFilterWellFormed(mdbean);
		} else if (type == "action") { 
			return isActionWellFormed(mdbean);
		} else if (type == "security-role") { 
			return isSecurityRoleWellFormed(mdbean);
		}
		return false;
	}
	
	/**
	 * <p>Filter elements must either provide a reference to an incoming or 
	 * outgoing Filter class (not necessarily both), or it must provide a 
	 * depends parameter which in turn must be a list of other filter elements 
	 * separated by commas.</p>
	 * 
	 * @param mdbean
	 * @return
	 */
	private boolean isFilterWellFormed(MDBean mdbean) {
		String name = (String)mdbean.getAttribute("name");
		String incoming = (String)mdbean.getAttribute("incoming");
		String outgoing = (String)mdbean.getAttribute("outgoing");
		String depends = (String)mdbean.getAttribute("depends");
		String managed = (String)mdbean.getAttribute("managed");
		String version = (String)mdbean.getAttribute("version");
		
		// Filter must have a name.
		if(name == "null") {
			log.info("Descriptor error for " + name + ". Abandoned by loader.");
			return false;
		}
		
		// must have either "incoming" or "outgoing" OR, it must have a "depends" string.
		if((incoming == "null" && outgoing == "null") && depends == "null") {
			log.info("Descriptor error for " + name + ". Abandoned by loader.");
			return false;
		}
		return true;
	}
	
	/**
	 * <p>Action elements must provide a "name" attribute which ends with the
	 * appropriate suffix (we recommend ".exec").  They must provide the 
	 * class name of their action class ("com.mygroup.actions.Example"), 
	 * which must in turn extend shocks.framework.action.ActionSupport.</p>
	 * 
	 * <p>Action elements can optionally provide a "permission" parameter which
	 * points toward a "security-role" element.  The "security-role" elements
	 * directly conform to the security-roles of the CMP container.  This
	 * allows simple action-level security on a fine-grained basis.</p>
	 * 
	 * <p>Action elements may also optionally add a reference to a filter 
	 * element. This will instruct the container to build a filter chain 
	 * which screens the flow of control on its way to and from the action.</p>
	 * 
	 * <p>Action elements which are going to be shared between application 
	 * modules must present a "managed" attribute set to "true". The 
	 * application assumes actions are not managed by default.</p>
	 * 
	 * <p>Finally, action elements can provide an optional "version" attribute
	 * which can be used by the MetadataRepository to determine the priority
	 * of entries in case of a conflict.</p>
	 * 
	 * @param mdbean
	 * @return
	 */
	private boolean isActionWellFormed(MDBean mdbean) {
		String name = (String)mdbean.getAttribute("name");
		String className = (String)mdbean.getAttribute("class");
		String permission = (String)mdbean.getAttribute("permission");
		String filter = (String)mdbean.getAttribute("filter");
		String managed = (String)mdbean.getAttribute("managed");
		String version = (String)mdbean.getAttribute("version");
		
		// Action must have a name and a className.
		if(name == "null" || className == "null") {
			log.info("Descriptor error for " + name + ". Abandoned by loader.");
			return false;
		}
		return true;
	}
	
	/**
	 * Will determine whether the security-role element is well formed.
	 * 
	 * @param mdbean
	 * @return boolean
	 */
	private boolean isSecurityRoleWellFormed(MDBean mdbean) {
		String name = (String)mdbean.getAttribute("name");
		String includes = (String)mdbean.getAttribute("includes");
		
		if(name == "null") {
			log.info("Descriptor error for " + name + ". Abandoned by loader.");
			return false;
		} 
		return true;
	}
	
	/**
	 * This compares the version strings and returns the highest value.  We
	 * recommend that people use version numbers (i.e. "1.0", "1.1.1", etc).
	 * 
	 * @param elementVersion
	 * @return String
	 */
	public String checkVersion(String elementVersion) {
		String defaultVersion = (String)ctx.getAttribute("shocks.defaultVersion");
		if(defaultVersion == null) {
			throw new Error("Workflow descriptor failed to provide the required " +
				"version attribute of the workflow element.");
		} else if(elementVersion != null || elementVersion != "null") {
			int i = defaultVersion.compareTo(elementVersion);
			if(i < 0) {
				//log.debug("Element version is greater.");
				return elementVersion;
			} 
		}
		return defaultVersion;
	}
	
	/**
	 * This method ensures that null references in the Descriptor are replaced
	 * with "null" Strings instead.
	 *  
	 * null references and empty Strings in the metadata should be replaced 
	 * with the "null" String.  Further validation of the metadata is performed
	 * by the validateMDBean() method.
	 * 
	 * @param value
	 * @return String
	 */
	public String checkNullString(String value) {
		if(value == null || value.equals("")) {
			return "null";
		}
		return value;
	}
	
	/**
	 * This compares the version strings and returns the highest value.  We
	 * recommend that people use version numbers (i.e. "1.0", "1.1.1", etc).
	 * 
	 * @param managed
	 * @return String representing the correct version input
	 */
	public String checkManaged(String managed) {
		if(!managed.equals("true")){
			managed = "false";
		}
		return managed;
	}
}
