/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.workflow;

import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;

import org.apache.log4j.Logger;

import shocks.dawp.*;
import shocks.framework.meta.*;
import shocks.framework.meta.repository.*;
import shocks.framework.workflow.sequence.*;

/**
 * This class uses the command string from the request URI to retrieve a
 * workflow sequence from the WorkflowRepository.  It then forwards control to 
 * the workflow sequence, after which it releases the sequence back into the
 * repository. 
 * 
 */
public final class WorkflowController {
	
	/**
	 * The forward method accepts a String as its first parameter which must be 
	 * a command string signifying the next workflow sequence corresponding to
	 * the metadata in the "shocks-workflow.xml" file. 
	 * 
	 * For example, if you've delegated the ".exec" suffix for all of your 
	 * Action sequences, a valid command string should end with ".exec".
	 * 
	 * Future support for display or ".view" components is pending.  For the
	 * moment, you can forward to .jsps or whatever other display component
	 * framework you would like to use.
	 * 
	 */
	public static void forward(
		String nextCommand, 
		HttpServletRequest request, 
		HttpServletResponse response,
		ServletContext ctx) {
		
		Logger log = Logger.getLogger("WorkflowController");
		String actionSuffix = (String)ctx.getAttribute("shocks.actionSuffix");
		String displaySuffix = (String)ctx.getAttribute("shocks.viewSuffix");
		
		MetadataRepository mdr = MetadataRepositoryFactory.getRepository(ctx);
		SequenceRepository sequenceRepository = SequenceRepositoryFactory.getRepository(ctx);
		
		if(nextCommand.endsWith("." + actionSuffix)) {
			DataSource metadata = mdr.leaseDataSource("action", nextCommand);
			log.debug("Executing Action: " + nextCommand);
			
			Workflow sequence = sequenceRepository.leaseWorkflow(metadata);
			DataSource ds = sequence.forward(request, response, ctx);
			String resultName = (String)ds.getAttribute("result");
			String next = ((MDBean)metadata).getResult(resultName);
			WorkflowController.forward(next, request, response, ctx);
			sequenceRepository.release(sequence);
			
		} else if(nextCommand.endsWith("." + displaySuffix)) {
			log.info("Shocks does not yet support display sequences.");
			
		} else {
			log.debug("Forwarding " + nextCommand + " to the web container.");
			RequestDispatcher dispatcher = request.
				getRequestDispatcher(nextCommand);
			try {
				dispatcher.forward(request, response);
			} catch(IOException e) {
				e.printStackTrace();
			} catch(ServletException e) {
				e.printStackTrace();
			}
		}
	}
}
