/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.workflow.filter;

import javax.servlet.*;
import org.apache.log4j.*;
import javax.management.*;
import shocks.framework.util.*;
import shocks.framework.workflow.filter.repository.*;

public class FilterRepositoryFactory {
	private static Logger log = Logger.getLogger("Shocks][FRFactory");

	/**
	 * This will search the MBeanServer for a registered repository. If it does
	 * not find one, it will create a new one and register it.  If during the
	 * course of registration it finds one has since been created, it will 
	 * return the one registered to the system.
	 * 
	 * Note that this method does not attempt to load or staff the repository
	 * in any way--it simply creates and registers a new one or returns a 
	 * reference to an existing one.
	 * 
	 * @return FilterRepository
	 */
	public static FilterRepository getRepository(ServletContext ctx) {
		MBeanServer server = MBeanServerBridge.getServer();
		return new FilterRepositoryProxy(getObjectName(server, ctx));
	}
	
	/**
	 * This method loads the FilterRepository and returns a reference to
	 * it.
	 * 
	 * @param ctx
	 * @return FilterRepository
	 */
	public static FilterRepository load(ServletContext ctx) {
		// Get referenes to the MBeanServer and the repository's ObjectName
		MBeanServer server = MBeanServerBridge.getServer();
		ObjectName objName = getObjectName(server, ctx);
		
		// Staff the repository with the metadata components
		CentralFilterRepositoryLoader.load(ctx);
		return new FilterRepositoryProxy(objName);
	}
	
	private static ObjectName getObjectName(MBeanServer server, ServletContext ctx) {
		String appName = (String)ctx.getAttribute("shocks.appName");
		String name = "Shocks:application=" + appName + ",name=FilterRepository";
		ObjectName objName = null;
		
		try {
			objName = new ObjectName(name);
			// see if repository is already registered.
			ObjectInstance instance = server.getObjectInstance(objName);
		} catch (InstanceNotFoundException e) {
			// there is no repository.  create a new one.
			registerNewRepository(server, objName);
			return getObjectName(server, ctx);
		} catch (MalformedObjectNameException e) {
			e.printStackTrace();
		}
		return objName;
	}
	
	/**
	 * Create a new, empty repository and register it with the MBeanServer
	 * 
	 */
	private static void registerNewRepository(
		MBeanServer server,
		ObjectName objName) {
		try {
			FilterRepository repository = new CentralFilterRepository();
			server.registerMBean(repository, objName);
			log.debug("Created and registered the CentralFilterRepository");
		} catch (InstanceAlreadyExistsException e) {
			log.debug("Repository instance already exists.  Try again.");
		} catch (NotCompliantMBeanException e) {
			e.printStackTrace();
		} catch (MBeanRegistrationException e) {
			e.printStackTrace();
		} catch (NullPointerException e) {
			e.printStackTrace();
		}
	}
}
