/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.workflow.filter.repository;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.management.*;

import shocks.client.*;
import shocks.dawp.*;
import shocks.framework.util.*;
import shocks.framework.workflow.filter.*;
import shocks.framework.meta.*;
import org.apache.log4j.*;

public class CentralFilterRepositoryLoader {
	public static void load(ServletContext ctx){
		Logger log = Logger.getLogger("Shocks][CFRLoader");
		MBeanServer server = MBeanServerBridge.getServer();
		MetadataRepository mdr = MetadataRepositoryFactory.getRepository(ctx);
		Set filters = mdr.getTypeSet("filter");
		
		Iterator iter = filters.iterator();
		while(iter.hasNext()) {
			DataSource metadata = (DataSource)iter.next();
			if(((String)metadata.getAttribute("depends")).equals("null")) {

				FilterRepository repository = 
					FilterRepositoryFactory.getRepository(ctx);
				Filter filter = (Filter)repository.leaseWorkflow(metadata);
				if(filter == null) {
					filter = registerNewFilter(metadata, repository, ctx);
				}
			} else {
				log.debug("");
				log.debug((String)metadata.getAttribute("name") + " filter" +
					" has dependencies & needn't be loaded.");
			}
		}
	}
	
	private static Filter registerNewFilter(
		DataSource metadata,
		FilterRepository repository,
		ServletContext ctx) {
		
		Logger log = Logger.getLogger("Shocks][CFRLoader");
		Filter filter = buildFilter(metadata, ctx);
		if(filter != null) {
			repository.addWorkflow(filter);
			return (Filter)repository.leaseWorkflow(metadata);
		}
		return null;
	}
	
	private static Filter buildFilter(
		DataSource metadata, 
		ServletContext ctx) {
		
		Logger log = Logger.getLogger("Shocks][CFRLoader");
		
		String incomingName = (String)metadata.getAttribute("incoming");
		String outgoingName = (String)metadata.getAttribute("outgoing");
		String name = (String)metadata.getAttribute("name");
		
		FilterSupport incomingAction = null;
		FilterSupport outgoingAction = null;
		
		if(incomingName.equals("null") || incomingName == null){
			log.debug("incoming is null for " + name);
			incomingAction = new FilterCap();
		} else {
			log.debug("incoming is not null for " + name + ". It is " + incomingName);
			incomingAction = getFilterSupport(metadata, incomingName, "incoming");
		}
		
		if(outgoingName.equals("null") || outgoingName == null){
			log.debug("outgoing is null for " + name);
			outgoingAction = new FilterCap();
		} else {
			log.debug("outgoing is not null for " + name + ". It is " + outgoingName);
			outgoingAction = getFilterSupport(metadata, outgoingName, "outgoing");
		}

		return new Filter(metadata, incomingAction, outgoingAction);
	}
	
	private static FilterSupport getFilterSupport(
		DataSource metadata, 
		String className, 
		String status) {
			
		Logger log = Logger.getLogger("Shocks][CFRLoader");
		FilterSupport filterSupport = null;
		
		try {
			ClassLoader loader = CentralFilterRepositoryLoader.
				class.getClassLoader();
				
			Class c = loader.loadClass(className);
			filterSupport = (FilterSupport)c.newInstance();
			filterSupport.setFilterMetadata(metadata, status);
			log.debug("FilterSupport class " + className + " = " + filterSupport );
			
		} catch (ClassNotFoundException e) {
			log.warn("Couldn't load " + className + " in ClassLoader");
		
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return filterSupport;
	}
	
	private static class FilterCap extends FilterSupport {
		public String execute(
			HttpServletRequest request, 
			HttpServletResponse response) {
			
			return SUCCESS;
		}
	}
}