/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */
 
package shocks.framework.workflow.filter.repository;

import java.util.*;
import org.apache.log4j.*;
import shocks.dawp.*;

/**
 * This class manages versioning for the filter repository.
 */
public class FilterVersionCache {
	private HashMap versions = new HashMap();
	private String currentVersion = null;
	private Logger log = Logger.getLogger("Shocks][FilterVersionCache");
	
	public FilterVersionCache(Workflow instance){
		add(instance);
	}
	
	/**
	 * Get the instance pool with the specified version and return a readied
	 * DataSource instance.
	 * 
	 * @return Workflow
	 */
	public Workflow lease(String version) {
		//log.info("lease(" + version + ")");
		if (versions.containsKey(version)) {
			return ((FilterInstancePool)versions.get(version)).lease();
		}
		return null;
	}

	/**
	 * 
	 * @param instance
	 */
	public synchronized void add(Workflow instance){
		String version = ((Filter)instance).VERSION;
		if(currentVersion() != null) {
			log.debug("incoming version = " + version + 
				", currentVersion = " + currentVersion());
			int i = version.compareTo(currentVersion());
			if(i < 0) {
				log.debug("incoming version is older.");
				return;
			}
		}
		log.debug("Incoming version is newer.");
		setCurrentVersion(version);
		versions.put(version, new FilterInstancePool(instance));
	}
	
	public synchronized void release(Workflow instance) {
		String version = ((Filter)instance).VERSION;
		((FilterInstancePool)versions.get(version)).release(instance);
	}
	
	private synchronized String currentVersion() {
		return currentVersion;
	}
	
	private synchronized void setCurrentVersion(String version) {
		currentVersion = version;
	}
	
	private FilterInstancePool newestPool() {
		return (FilterInstancePool)versions.get(currentVersion());
	}
}
