/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.workflow.sequence;

import javax.servlet.*;
import javax.management.*;

import org.apache.log4j.*;
import shocks.framework.util.*;
import shocks.framework.workflow.sequence.repository.*;

/**
 * This class is used to get a reference to the WorkflowController or the
 * WorkflowRepository.  Each of its two methods require a reference to the
 * ServletContext object.
 * 
 * The implementation details of the WorkflowRepository and WorkflowController
 * are kept from the client so that they can be reimplemented at a later time
 * if it becomes necessary.
 */
public class SequenceRepositoryFactory {
	private static Logger log = Logger.getLogger("Shocks][SRFactory");
	
	public static SequenceRepository getRepository(ServletContext ctx) {
		MBeanServer server = MBeanServerBridge.getServer();
		return new SequenceRepositoryProxy(getObjectName(server, ctx));
	}
	
	public static SequenceRepository load(ServletContext ctx) {
		// Get referenes to the MBeanServer and the repository's ObjectName
		MBeanServer server = MBeanServerBridge.getServer();
		ObjectName objName = getObjectName(server, ctx);
		
		// Staff the repository with the action components
		CentralSequenceRepositoryLoader.load(ctx);
		return new SequenceRepositoryProxy(objName);
	}
	
	/**
	 * Return the ObjectName for the WorkflowRepository assigned to the working
	 * application.
	 * 
	 * @param server
	 * @param ctx
	 * @return
	 */
	private synchronized static ObjectName 
		getObjectName(MBeanServer server, ServletContext ctx) {
			
		String appName = (String) ctx.getAttribute("shocks.appName");
		String name = "Shocks:application=" + appName + ",name=WorkflowRepository";
		ObjectName objName = null;
		
		try {
			objName = new ObjectName(name);
			// see if repository is already registered.
			ObjectInstance instance = server.getObjectInstance(objName);
		} catch (InstanceNotFoundException e) {
			// there is no repository.  create a new one.
			registerNewRepository(server, objName, ctx);
			return getObjectName(server, ctx);
		} catch (MalformedObjectNameException e) {
			e.printStackTrace();
		}
		return objName;
	}
	
	/**
	 * Create a new WorkflowRepository and register it with the MBeanServer.
	 * @param server
	 * @param objName
	 * @param ctx
	 */
	private synchronized static void registerNewRepository(
		MBeanServer server,
		ObjectName objName,
		ServletContext ctx) {
		try {
			SequenceRepository repository = new CentralSequenceRepository();
			server.registerMBean(repository, objName);
		} catch (InstanceAlreadyExistsException e) {
			log.info("Repository instance already exists.  Try again.");
		} catch (NotCompliantMBeanException e) {
			e.printStackTrace();
		} catch (MBeanRegistrationException e) {
			e.printStackTrace();
		} catch (NullPointerException e) {
			e.printStackTrace();
		}
	}
}
