/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.workflow.sequence.repository;

import java.util.*;
import shocks.dawp.*;
import shocks.framework.workflow.sequence.*;

import org.apache.log4j.*;

public class CentralSequenceRepository
	implements SequenceRepository, CentralSequenceRepositoryMBean {
		
	private Map pools = new HashMap();
	private Logger log = Logger.getLogger("Shocks][CSR");
	
	/**
	 * Leases one instance of the appropriate sequence.  The sequence released
	 * will default to the latest version entered into the system, unless the
	 * admin opts to override the version controls, in which case they can
	 * request a list of sequence versions currently in the system and choose a
	 * version to revert to. Unless this override is turned off, the system
	 * will load but not adopt the behavior of the new sequences.
	 * 
	 * At this time it makes no sense to support any other type of call than an
	 * action call.  The next item on the list will probably be display calls,
	 * but at this time all display pieces are pluggable through the workflow
	 * descriptor, so the matter is moot.
	 */
	public Workflow leaseWorkflow(DataSource metadata) {
		String type = (String)metadata.getAttribute("type");
		if (type.equals("action")) {
			String name = (String)metadata.getAttribute("name");
			if (pools.containsKey(name)) {
				return pool(name).lease();
			} else {
				log.debug("no pool for " + name);
			}
		} else {
			log.error("Can't use " + type + " metadata.");
		}
		return null;
	}

	public void release(Workflow instance) {
		String name = ((Sequence)instance).NAME;
		pool(name).release(instance);
	}
	
	/**
	 * The mechanism by which sequence components are added to the system. 
	 * This method checks to see if a sequence exists with the same name as the
	 * incoming component.  If there is one with the same name, the method will
	 * compare the versions of the existant and the incoming component.  If the
	 * incoming component's version tag is equal to or lower than the highest
	 * version in the system, it will not load and an error will be logged.
	 * 
	 * If the incoming version is higher, however, it will be loaded into the
	 * system and an instance pool will be created for it.
	 */
	public void addWorkflow(Workflow sequence) {
		String name = ((Sequence)sequence).NAME;
		String version = ((Sequence)sequence).VERSION;
		if (pools.containsKey(name)) {
			pool(name).add(sequence);
		} else {
			SequenceVersionCache pool = new SequenceVersionCache(sequence);
			pools.put(name, pool);
		}
		log.info("Loaded sequence, name = " + name + ", version = " + 
			((Sequence)sequence).VERSION);
	}
	
	public String getVersion(DataSource metadata) {
		//TODO: implement this method.
		return "";
	}
	
	public void setCurrentVersion(DataSource metadata, String version) {
		//TODO: implement this method.
	}
	
	public SortedSet getVersions(DataSource metadata) {
		//TODO: implement this method.
		return null;
	}

	private SequenceVersionCache pool(String name) {
		return (SequenceVersionCache)pools.get(name);
	}
}