/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.framework.workflow.sequence.repository;

import shocks.dawp.*;
import java.util.*;
import org.apache.log4j.*;

public class SequenceVersionCache {
	private HashMap versions = new HashMap();
	private String currentVersion = null;
	private Logger log = Logger.getLogger("Shocks][SequenceVersionCache");
	private final String NAME;
	
	public SequenceVersionCache(Workflow sequence){
		this.NAME = ((Sequence)sequence).NAME;
		add(sequence);
	}
	
	/**
	 * Get the instance pool with the most recent version and return a readied
	 * Workflow component.
	 * 
	 * @return Workflow sequence
	 */
	public Workflow lease() {
		if (newestPool() != null) {
			return newestPool().lease();
		}
		return null;
	}

	/**
	 * Adds a workflow instance to the SequenceInstancePool provided there is
	 * no sequence existing with the same or lesser version.
	 * 
	 * @param sequence to be added to the repository
	 */
	public synchronized void add(Workflow sequence){
		String version = ((Sequence)sequence).VERSION;
		if(getCurrentVersion() != null) {
			int i = version.compareTo(getCurrentVersion());
			if(i < 0) {
				// A newer version already exists.  Do nothing.
				return;
			}
		}
		// Incoming version is newer.
		versions.put(version, new SequenceInstancePool(sequence));
		setCurrentVersion(version);
	}
	
	public synchronized void release(Workflow sequence) {
		String version = ((Sequence)sequence).VERSION;
		((SequenceInstancePool)versions.get(version)).release(sequence);
	}
	
	/**
	 * This method returns the current operating version of the sequence
	 * instance pool.
	 * 
	 * While I don't usually condone package-private methods, this needn't
	 * be used by any class outside of this package.
	 * 
	 * @return
	 */
	synchronized String getCurrentVersion() {
		return currentVersion;
	}
	
	/**
	 * Set the current operating version of the sequence instance pooling 
	 * mechanism, provided that version exists presently in the system.  If 
	 * the sequence version does not exist, an error will be logged and 
	 * no change will occur.
	 * 
	 * @param version
	 */
	synchronized void setCurrentVersion(String version) {
		if(versions.containsKey(version)) {
			currentVersion = version;
		} else {
			log.error("No " + NAME + " sequence of version " + version + 
				" exists");
		}
	}
	
	/**
	 * Return the instance pool with the highest version.
	 * @return
	 */
	private SequenceInstancePool newestPool() {
		return (SequenceInstancePool)versions.get(getCurrentVersion());
	}
}
